%This code is associated with the publication:
%"Harmonic radar tracking reveals that honeybee drones navigate between multiple aerial leks"
%by: Joseph L. Woodgate, James C. Makinson, Natacha Rossi, Ka S. Lim,
%Andrew M. Reynolds, Christopher J. Rawlings, Lars Chittka 
%2021, iScience
%
%This script takes radar tracking data and derives all variables used in
%analysing this data in the publication
%The analyses are conducted using another script:
%"HoneybeeDroneTracking_Analysis.m"
%
%All code written  by Joseph Woodgate, 2021, except the following functions
%called during execution of the script:
%ang2rad, rad2ang, circ_r, by Philipp Berens, 2009; part of the Circular
%Statistics Toolbox for Matlab
%vreckon, by Michael Kleder, 2007
%
%This code requires dataset:
%"HoneybeeDroneTracking_UnprocessedData.mat"
%archived at:
%

%The following variables were manually extracted from data recorded by the
%harmonic radar and are basic to the analysis:
% DroneFlightPaths.DateTime
% DroneFlightPaths.X
% DroneFlightPaths.Y
% QueenFlightPaths.DateTime
% QueenFlightPaths.X
% QueenFlightPaths.Y

%The following variables were extracted from contemporaneous notes taken as the
%study was conducted and are basic to the analysis:
% DroneFlightSegments.SegmentNo
% DroneFlightSegments.BeeID
% DroneFlightSegments.Year
% DroneFlightSegments.StartLocation
% DroneFlightSegments.EndLocation
% DroneFlightSegments.Orientation
% DroneFlightPaths.SegmentNo
% QueenFlightSegments.SegmentNo
% QueenFlightSegments.BeeID
% QueenFlightSegments.Year
% QueenFlightSegments.StartLocation
% QueenFlightSegments.EndLocation
% QueenFlightPaths.SegmentNo
% Hives.Name
% Hives.X
% Hives.Y
% Hives.GPSLat
% Hives.GPSLong


%All other variables were derived from the basic variables using the following
%code:

%Process data to derive variables:

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% DRONES

%%
%Flight Segment number nested within drone ID
[udrones, ~, thisdrone]=unique(DroneFlightSegments.BeeID);
DroneFlightSegments.BeeFlightNo=NaN(size(DroneFlightSegments.SegmentNo));

for uind=1:length(udrones)
    if udrones(uind)<1000
    DroneFlightSegments.BeeFlightNo(thisdrone==uind)=(1:length(find(thisdrone==uind)))';
    end
end
%%
%Flight type: 0=neither starts or ends at hive; 1=complete flight, starts and
%ends at same hive; 2=starts at hive but return portion missing; 3=start
%portion missing but returns to hive
DroneFlightSegments.FlightType=NaN(size(DroneFlightSegments.SegmentNo));
DroneFlightSegments.FlightType(DroneFlightSegments.StartLocation==DroneFlightSegments.EndLocation & DroneFlightSegments.StartLocation>0 & DroneFlightSegments.EndLocation>0)=1;
DroneFlightSegments.FlightType(DroneFlightSegments.StartLocation>0 & DroneFlightSegments.StartLocation~=DroneFlightSegments.EndLocation)=2;
DroneFlightSegments.FlightType(DroneFlightSegments.StartLocation==0 & DroneFlightSegments.EndLocation>0)=3;
DroneFlightSegments.FlightType(DroneFlightSegments.StartLocation==0 & DroneFlightSegments.EndLocation==0)=0;

%%
%Work out GPS coordinates of all tracking data
[DroneFlightPaths.GPSLat, DroneFlightPaths.GPSLong]=ConvertCart2GPS(DroneFlightPaths.X, DroneFlightPaths.Y, Hives.X(1), Hives.Y(1), Hives.GPSLat(1), Hives.GPSLong(1));

%%
%Distances, times elapsed, speeds for each flight path
%Total distances, durations, start and end times, mean speeds for each flight segment; 
DroneFlightPaths.BeeID=NaN(size(DroneFlightPaths.SegmentNo));
DroneFlightPaths.SecondsCumulative=NaN(size(DroneFlightPaths.SegmentNo));
DroneFlightPaths.SecondsStep=NaN(size(DroneFlightPaths.SegmentNo));
DroneFlightPaths.DistanceStep=NaN(size(DroneFlightPaths.SegmentNo));
DroneFlightPaths.DistanceCumulative=NaN(size(DroneFlightPaths.SegmentNo));
DroneFlightPaths.SpeedStep=NaN(size(DroneFlightPaths.SegmentNo));
DroneFlightSegments.DataPointCount=NaN(size(DroneFlightSegments.SegmentNo));
DroneFlightSegments.StartTime=repmat(datetime(2000,1,1,1,0,0), size(DroneFlightSegments.SegmentNo));
DroneFlightSegments.EndTime=repmat(datetime(2000,1,1,1,0,0), size(DroneFlightSegments.SegmentNo));
DroneFlightSegments.Duration=NaN(size(DroneFlightSegments.SegmentNo));
DroneFlightSegments.DistanceTotal=NaN(size(DroneFlightSegments.SegmentNo));
DroneFlightSegments.DistanceMax=NaN(size(DroneFlightSegments.SegmentNo));
DroneFlightSegments.SpeedMean=NaN(size(DroneFlightSegments.SegmentNo));
DroneFlightSegments.SpeedSD=NaN(size(DroneFlightSegments.SegmentNo));
%
useg=unique(DroneFlightPaths.SegmentNo);
for sind=1:length(useg)
     thisseg=find(DroneFlightPaths.SegmentNo==useg(sind));
    DroneFlightPaths.BeeID(thisseg)=repmat(DroneFlightSegments.BeeID(useg(sind)), length(thisseg), 1);
    DroneFlightPaths.SecondsStep(thisseg)=[0; seconds(diff(DroneFlightPaths.DateTime(thisseg)))];
    DroneFlightPaths.SecondsCumulative(thisseg)=seconds(DroneFlightPaths.DateTime(thisseg)- DroneFlightPaths.DateTime(thisseg(1)));
     xdiff=[0; diff(DroneFlightPaths.X(thisseg))]  ; 
     ydiff=[0; diff(DroneFlightPaths.Y(thisseg))]  ; 
     [~,rdiff]=cart2pol(xdiff, ydiff);
    DroneFlightPaths.DistanceStep(thisseg)=rdiff;
    DroneFlightPaths.DistanceCumulative(thisseg)=cumsum(rdiff);
    DroneFlightPaths.SpeedStep(thisseg)=DroneFlightPaths.DistanceStep(thisseg)./DroneFlightPaths.SecondsStep(thisseg);
    
    DroneFlightSegments.DataPointCount(sind)=length(thisseg);
    DroneFlightSegments.DistanceTotal(sind)=DroneFlightPaths.DistanceCumulative(thisseg(end));
     xdiffstart=DroneFlightPaths.X(thisseg)-DroneFlightPaths.X(thisseg(1));
     ydiffstart=DroneFlightPaths.Y(thisseg)-DroneFlightPaths.Y(thisseg(1));
     [~, rdiffstart]=cart2pol(xdiffstart, ydiffstart);
    DroneFlightSegments.DistanceMax(sind)=max(rdiffstart);
    DroneFlightSegments.StartTime(sind)=DroneFlightPaths.DateTime(thisseg(1));
    DroneFlightSegments.EndTime(sind)=DroneFlightPaths.DateTime(thisseg(end));
    
    DroneFlightSegments.SpeedMean(sind)=mean(DroneFlightPaths.SpeedStep(thisseg(2:end)));
    DroneFlightSegments.SpeedSD(sind)=std(DroneFlightPaths.SpeedStep(thisseg(2:end)));
  
end
DroneFlightSegments.Duration=DroneFlightSegments.EndTime-DroneFlightSegments.StartTime;


%%
%Detect convoluted sections of each flightpath 

%Parameters for convoluted flight detection
vectorwindow=7*3; %21s, 3 revolutions of the radar
minstraightness=0.7; %minimum vector length
maxstepduration=12; %maximum gap permissible between datapoints
maxsteplength=40; %maximum movement permitted between consecutive datapoints
mindps=7; %minimum number of datapoints allowed in a convoluted section
stepandor=1; %1=only exclude datapoints that exceed BOTH maxstepduration and maxsteplength; 2=eclude datapoints that exceed EITHER maxstepduyration OR maxsteplength

%Create new variables to store convoluted leg data
DroneFlightPaths.ConvolutedSectionNoGlobal=zeros(size(DroneFlightPaths.SegmentNo));
DroneFlightPaths.ConvolutedSectionNoWithinSegment=zeros(size(DroneFlightPaths.SegmentNo));
DroneFlightSegments.ConvolutedSectionCount=NaN(size(DroneFlightSegments.SegmentNo));
cfgcounter=0;

useg=unique(DroneFlightPaths.SegmentNo);
for sind=1:length(useg)
     cfscounter=0;
     thisseg=find(DroneFlightPaths.SegmentNo==useg(sind));
     datainx=DroneFlightPaths.X(thisseg);
     datainy=DroneFlightPaths.Y(thisseg);
     dataint=DroneFlightPaths.SecondsCumulative(thisseg);
    %Find starts and ends of convoluted legs
    [convstartind, convendind]=DCAFlightDetector(datainx, datainy, dataint, vectorwindow, minstraightness, maxstepduration, maxsteplength, mindps, stepandor); 
    
    if isempty(convstartind)==0
        for cind=1:length(convstartind)
             cfgcounter=cfgcounter+1;
             cfscounter=cfscounter+1;
            DroneFlightPaths.ConvolutedSectionNoGlobal(thisseg(convstartind(cind):convendind(cind)))=cfgcounter;
            DroneFlightPaths.ConvolutedSectionNoWithinSegment(thisseg(convstartind(cind):convendind(cind)))=cfscounter;
        end
            DroneFlightSegments.ConvolutedSectionCount(sind)=cfscounter;
    else
        DroneFlightSegments.ConvolutedSectionCount(sind)=0;
    end
end

%%
%Details of each convoluted section of flight
 noconvflights=max(DroneFlightPaths.ConvolutedSectionNoGlobal);
DroneConvolutedFlights.ConvolutedSectionNoGlobal=NaN(noconvflights,1);
DroneConvolutedFlights.ConvolutedSectionNoWithinSegment=NaN(noconvflights,1);
DroneConvolutedFlights.SegmentNo=NaN(noconvflights,1);
DroneConvolutedFlights.BeeID=NaN(noconvflights,1);
DroneConvolutedFlights.StartTime=repmat(datetime(2000,1,1,1,0,0), noconvflights,1);
DroneConvolutedFlights.EndTime=repmat(datetime(2000,1,1,1,0,0), noconvflights,1);
DroneConvolutedFlights.CoMX=NaN(noconvflights,1);
DroneConvolutedFlights.CoMY=NaN(noconvflights,1);
DroneConvolutedFlights.DistanceTotal=NaN(noconvflights,1);
DroneConvolutedFlights.SpeedMean=NaN(noconvflights,1);
DroneConvolutedFlights.SpeedSD=NaN(noconvflights,1);
DroneConvolutedFlights.FlightPathIndices=cell(noconvflights,1);
DroneConvolutedFlights=struct2table(DroneConvolutedFlights);

for cind=1:noconvflights
     thisseg=find(DroneFlightPaths.ConvolutedSectionNoGlobal==cind);
     tempx=DroneFlightPaths.X(thisseg);
     tempy=DroneFlightPaths.Y(thisseg);
     dx=diff(tempx);
     dy=diff(tempy);
     [~,dr]=cart2pol(tempx,tempy);
    DroneConvolutedFlights.ConvolutedSectionNoGlobal(cind)=cind;
    DroneConvolutedFlights.ConvolutedSectionNoWithinSegment(cind)=DroneFlightPaths.ConvolutedSectionNoWithinSegment(thisseg(1));
    DroneConvolutedFlights.SegmentNo(cind)=DroneFlightPaths.SegmentNo(thisseg(1));
    DroneConvolutedFlights.BeeID(cind)=DroneFlightPaths.BeeID(thisseg(1));
    DroneConvolutedFlights.StartTime(cind)=DroneFlightPaths.DateTime(thisseg(1));
    DroneConvolutedFlights.EndTime(cind)=DroneFlightPaths.DateTime(thisseg(end));
    DroneConvolutedFlights.CoMX(cind)=mean(tempx);
    DroneConvolutedFlights.CoMY(cind)=mean(tempy);
    DroneConvolutedFlights.DistanceTotal(cind)=sum(dr);
    DroneConvolutedFlights.SpeedMean(cind)=mean(DroneFlightPaths.SpeedStep(thisseg(2:end)));
    DroneConvolutedFlights.SpeedSD(cind)=std(DroneFlightPaths.SpeedStep(thisseg(2:end)));
    DroneConvolutedFlights.FlightPathIndices{cind}=thisseg;
end
    DroneConvolutedFlights.Duration=DroneConvolutedFlights.EndTime-DroneConvolutedFlights.StartTime;

    DroneFlightSegments.DurationConvolutedSections=DroneFlightSegments.Duration;
useg=unique(DroneFlightPaths.SegmentNo);
for sind=1:length(useg)
   thisseg=find(DroneConvolutedFlights.SegmentNo==useg(sind)); 
   DroneFlightSegments.DurationConvolutedSections(sind)=sum(DroneConvolutedFlights.Duration(thisseg));
end

%%
%Cluster analysis of convoluted flight sections to detect candidate drone
%Congregation Areas

%Parameters for cluster analysis
cutoff=8; %cutoff for clustering algorithm
minhivedist=50; %Exclude conv legs whose CoM is closer to a hive than this
minflightno=10; %DCAs have to have at least this many different flights contributing
hiveperiods=[datetime(2016,1,1,0,0,0); datetime(2017,1,1,0,0,0); datetime(2019,1,1,0,0,0); datetime(2019,12,31,23,59,59)];
hiveperiodhives={[1,2,3], [1,2,3,4], [1,2,3,5]};

%Calculate distance of each datapoint from the hives. Exclude any that were
%within 50m of any hive
hivedistx=repmat(DroneConvolutedFlights.CoMX,1,length(Hives.X))-repmat((Hives.X)', length(DroneConvolutedFlights.CoMX),1);
hivedisty=repmat(DroneConvolutedFlights.CoMY,1,length(Hives.Y))-repmat((Hives.Y)', length(DroneConvolutedFlights.CoMY),1);
[~, hivedistr]=cart2pol(hivedistx, hivedisty);
hivedistcutoff=zeros(size(hivedistr));
hivedistcutoff(hivedistr<minhivedist)=1;

%Look at which hives active
hiveexclude=NaN(size(DroneConvolutedFlights.CoMX));
for hind=1:length(hiveperiodhives)
    thistime=find(DroneConvolutedFlights.StartTime>hiveperiods(hind) & DroneConvolutedFlights.StartTime<=hiveperiods(hind+1));
    hiveexclude(thistime)=sum(hivedistcutoff(thistime,hiveperiodhives{hind}),2);
end

%Find datapoints that are part of convoluted flight but not too close to
%any hive
inclsection=find(hiveexclude==0);
inclind=[];
for isind=1:length(inclsection)
   inclind=[inclind; DroneConvolutedFlights.FlightPathIndices{inclsection(isind)}]; 
end
    tX=DroneFlightPaths.X(inclind);
    tY=DroneFlightPaths.Y(inclind);
    tSegmentNo=DroneFlightPaths.SegmentNo(inclind);

%Assign each datapoint to a cluster    
    T = clusterdata([tX, tY], 'Criterion','distance','Cutoff', cutoff, 'Distance', 'euclidean');

%Exclude clusters with too few datapoints or within 50m of any hive
noclusters=max(T);
nodiffsegsincluster=zeros(noclusters,1);
clustercomx=zeros(noclusters,1);
clustercomy=zeros(noclusters,1);
for tind=1:noclusters
    thiscluster=find(T==tind);
    nodiffsegsincluster(tind)=length(unique(tSegmentNo(thiscluster)));
    clustercomx(tind)=mean(tX(thiscluster));
    clustercomy(tind)=mean(tY(thiscluster));
end
clusterdistx=repmat(clustercomx,1,length(Hives.X))-repmat((Hives.X)', length(clustercomx),1);
clusterdisty=repmat(clustercomy,1,length(Hives.Y))-repmat((Hives.Y)', length(clustercomy),1);
[~, clusterdistr]=cart2pol(clusterdistx, clusterdisty);
clusterdistcutoff=zeros(size(clusterdistr));
clusterdistcutoff(clusterdistr<minhivedist)=1;
clusterexclude=sum(clusterdistcutoff,2);
goodcluster=nodiffsegsincluster>=minflightno & clusterexclude==0;
goodclusterindices=find(goodcluster==1);

%%
%Details of clusters
    clusterorder=goodclusterindices([4;3;1;2]); %The clusters were named manually so the letter names A-D don't match the order in which the clusters were returned by 'clusterdata'. This harcoded nvariable puts them into the desired order
Congregations.Name={'A';'B';'C';'D'};
Congregations.IDNo=(1:4)';
Congregations.CoMX=clustercomx(clusterorder);
Congregations.CoMY=clustercomy(clusterorder);
Congregations.ClusterNo=clusterorder;
Congregations. FlightPathIndices=cell(size(Congregations.IDNo));
Congregations.HullVerticesXY=cell(size(Congregations.IDNo));
Congregations.HullPolygon=repmat(polyshape,size(Congregations.IDNo));
Congregations=struct2table(Congregations);

for cind=1:length(clusterorder)
    thiscluster=find(T==clusterorder(cind)); 
   Congregations.FlightPathIndices{cind}=inclind(thiscluster);
    clusterhullind=convhull(tX(thiscluster),tY(thiscluster));
   Congregations.HullVerticesXY{cind}=[tX(thiscluster(clusterhullind)),tY(thiscluster(clusterhullind))];
   Congregations.HullPolygon(cind)=polyshape(tX(thiscluster(clusterhullind)),tY(thiscluster(clusterhullind)));
end

%%
%Combine cluster and hive data to get locations of interest: we will check
%whether each convoluted flight occurs within 50m of each location
Locations.Name=[Congregations.Name; Hives.Name(1:3)];
Locations.IDNo=[Congregations.IDNo; Hives.IDNo(1:3)];
Locations.X=[Congregations.CoMX; Hives.X(1:3)];
Locations.Y=[Congregations.CoMY; Hives.Y(1:3)];
[Locations.GPSLat, Locations.GPSLong]=ConvertCart2GPS(Locations.X, Locations.Y, Hives.X(1), Hives.Y(1), Hives.GPSLat(1), Hives.GPSLong(1));
Locations.CongregationOrHive=[ones(size(Congregations.IDNo)); 2*ones(size(Hives.IDNo(1:3)))];
Locations=struct2table(Locations);

%%
%Add information on locations of convoluted flight to DroneConvolutedFlights 

%A section of convoluted flight was considered to take place at a
%congregation area or hive if its centre of mass was within 50m of the
%location
%It is possible for a CoM to be within 50m of both congregation A and hive 1, or
%both congregation D and  hive 3. If this occurs (as it does in 9/424
%convoluted sections), we assign the convoluted flight sectionj to the
%location it is closest to 
 locationdistx=repmat(DroneConvolutedFlights.CoMX,1,length(Locations.X))-repmat((Locations.X)', length(DroneConvolutedFlights.CoMX),1);
 locationdisty=repmat(DroneConvolutedFlights.CoMY,1,length(Locations.Y))-repmat((Locations.Y)', length(DroneConvolutedFlights.CoMY),1);
 [~, locationdistr]=cart2pol(locationdistx, locationdisty);
 [locationmindist, locationid]=min(locationdistr,[],2);
DroneConvolutedFlights.Location=locationid;
DroneConvolutedFlights.Location(locationmindist>=minhivedist)=0;
DroneConvolutedFlights.CongregationOrHive=zeros(size(DroneConvolutedFlights.CoMX));
DroneConvolutedFlights.CongregationOrHive(DroneConvolutedFlights.Location>0 & DroneConvolutedFlights.Location<5)=1;
DroneConvolutedFlights.CongregationOrHive(DroneConvolutedFlights.Location>=5)=2;

DroneFlightPaths.ConvolutedSectionLocation=zeros(size(DroneFlightPaths.ConvolutedSectionNoGlobal));
for cind=1:length(DroneConvolutedFlights.Location)
    DroneFlightPaths.ConvolutedSectionLocation(DroneConvolutedFlights.FlightPathIndices{cind})=DroneConvolutedFlights.Location(cind);
end

%%
%Components of position, velocity, acceleration, conditioned on distance
%from Location in X (South-North) and Y (West-East) directions
DroneFlightPaths.PositionXComponent=NaN(size(DroneFlightPaths.SegmentNo));
DroneFlightPaths.PositionYComponent=NaN(size(DroneFlightPaths.SegmentNo));
DroneFlightPaths.VelocityXComponent=NaN(size(DroneFlightPaths.SegmentNo));
DroneFlightPaths.VelocityYComponent=NaN(size(DroneFlightPaths.SegmentNo));
DroneFlightPaths.AccelerationXComponent=NaN(size(DroneFlightPaths.SegmentNo));
DroneFlightPaths.AccelerationYComponent=NaN(size(DroneFlightPaths.SegmentNo));
DroneConvolutedFlights.KurtosisPositionX=NaN(size(DroneConvolutedFlights.ConvolutedSectionNoGlobal));
DroneConvolutedFlights.KurtosisPositionY=NaN(size(DroneConvolutedFlights.ConvolutedSectionNoGlobal));
DroneConvolutedFlights.KurtosisVelocityX=NaN(size(DroneConvolutedFlights.ConvolutedSectionNoGlobal));
DroneConvolutedFlights.KurtosisVelocityY=NaN(size(DroneConvolutedFlights.ConvolutedSectionNoGlobal));


for cind=1:length(DroneConvolutedFlights.ConvolutedSectionNoGlobal)
    thissection=DroneConvolutedFlights.FlightPathIndices{cind};
    thissection=[thissection(1)-1; thissection; thissection(end)+1];
    thislocation=DroneFlightPaths.ConvolutedSectionLocation(thissection(2));
    if thislocation>0 %We can only calculate these variables for convoluted sectionjs taking place at a known location
         tempx=DroneFlightPaths.X(thissection);
         tempy=DroneFlightPaths.Y(thissection);
         tempt=DroneFlightPaths.DateTime(thissection);

         xrelativetolocation=tempx-Locations.X(thislocation);
         yrelativetolocation=tempy-Locations.Y(thislocation);

         dx=diff(xrelativetolocation);
         dy=diff(yrelativetolocation);
         dt=seconds(diff(DroneFlightPaths.DateTime(thissection)));
         velocityx=dx./dt;
         velocityy=dy./dt;

         dvx=diff(velocityx);
         dvy=diff(velocityy);
         accelerationx=dvx./dt(2:end);
         accelerationy=dvy./dt(2:end);

        DroneFlightPaths.PositionXComponent(thissection(2:end-1))=xrelativetolocation(2:end-1);
        DroneFlightPaths.PositionYComponent(thissection(2:end-1))=yrelativetolocation(2:end-1);
        DroneFlightPaths.VelocityXComponent(thissection(2:end-1))=velocityx(1:end-1);
        DroneFlightPaths.VelocityYComponent(thissection(2:end-1))=velocityy(1:end-1);
        DroneFlightPaths.AccelerationXComponent(thissection(2:end-1))=accelerationx;
        DroneFlightPaths.AccelerationYComponent(thissection(2:end-1))=accelerationy;
        %Velocities are calculated from the change in position over two
        %consecutive data points: i-1, i. If i is the first data point in a
        %flight segment there is no i-1 so we cannot calculate a velocity for
        %point i.
        %Accelerations are calculated as the change in flight speed across
        %three consecutive datapoints: the value associated with each data
        %point, i, is calculated from data from points i-1, i, i+1. If i is
        %the first data point in a flight segment, there is no i-1 so we cannot
        %calculate an acceleration for point i.
        if DroneFlightPaths.SegmentNo(thissection(1))~=DroneFlightPaths.SegmentNo(thissection(2))
            DroneFlightPaths.VelocityXComponent(thissection(2))=NaN;
            DroneFlightPaths.VelocityYComponent(thissection(2))=NaN;
            DroneFlightPaths.AccelerationXComponent(thissection(2))=NaN;
            DroneFlightPaths.AccelerationYComponent(thissection(2))=NaN;
        end
        %Accelerations are calculated as the change in flight speed across
        %three consecutive datapoints: the value associated with each data
        %point, i, is calculated from data from points i-1, i, i+1. If i is the last data point in a
        %flight segment there is no i+1 so we cannot calculate an acceleration for point i for the
        %same reason.
        if DroneFlightPaths.SegmentNo(thissection(end))~=DroneFlightPaths.SegmentNo(thissection(end)-1)
            DroneFlightPaths.AccelerationXComponent(thissection(end-1))=NaN;
            DroneFlightPaths.AccelerationYComponent(thissection(end-1))=NaN;
        end
        
    end
end

%%
%Kurtosis of position, velocity distributions
Locations.KurtosisPositionX=NaN(size(Locations.IDNo));
Locations.KurtosisPositionY=NaN(size(Locations.IDNo));
Locations.KurtosisVelocityX=NaN(size(Locations.IDNo));
Locations.KurtosisVelocityY=NaN(size(Locations.IDNo));

for lind=1:length(Locations.IDNo)
Locations.KurtosisPositionX(lind)=kurtosis(DroneFlightPaths.PositionXComponent(DroneFlightPaths.ConvolutedSectionLocation==lind));
Locations.KurtosisPositionY(lind)=kurtosis(DroneFlightPaths.PositionYComponent(DroneFlightPaths.ConvolutedSectionLocation==lind));
Locations.KurtosisVelocityX(lind)=kurtosis(DroneFlightPaths.VelocityXComponent(DroneFlightPaths.ConvolutedSectionLocation==lind));
Locations.KurtosisVelocityY(lind)=kurtosis(DroneFlightPaths.VelocityYComponent(DroneFlightPaths.ConvolutedSectionLocation==lind));
end

%%
%Detect visits to each congregation area
%Identify tracks in which either the bee performed a section of convoluted flight
%whose center of mass (mean coordinates of every data point) was within 50m
%of a cluster centre, or in which the bee stayed within 50m of a cluster
%centre for at least 21s (seven revolutions of the radar). This included
%periods in which the signal was lost, provided the positional fixes either
%side of the missing period were within 50m of a cluster centre: this can
%occur if the bee flies too high for the radar to detect      

%Parameters
DCADist=50;
DCATime=21;

%For every datapoint, determine whether it was within 50m of any location
nolocations=length(Locations.X);
locationdistx=repmat(DroneFlightPaths.X,1,nolocations)-repmat((Locations.X(1:nolocations))', length(DroneFlightPaths.X),1);
 locationdisty=repmat(DroneFlightPaths.Y,1,nolocations)-repmat((Locations.Y(1:nolocations))', length(DroneFlightPaths.Y),1);
 [~, locationdistr]=cart2pol(locationdistx, locationdisty);
 [locationmindist, locationid]=min(locationdistr,[],2);
DroneFlightPaths.Location=locationid;
DroneFlightPaths.Location(locationmindist>=DCADist)=0;

possvisitlocation=DroneFlightPaths.Location;
possvisitlocation(DroneFlightPaths.ConvolutedSectionLocation~=0)=DroneFlightPaths.ConvolutedSectionLocation(DroneFlightPaths.ConvolutedSectionLocation~=0);

atlocation=zeros(length(possvisitlocation),nolocations);
for lind=1:nolocations
    atlocation(possvisitlocation==lind,lind)=1;
end
dlocation=diff(atlocation);

%Find the start and end of each potential visit
    possstarts=[];
    possends=[];
    posslocation=[];
for lind=1:nolocations
    tempstarts=find(dlocation(:,lind)==1)+1;
    tempends=find(dlocation(:,lind)==-1);
            if isempty(tempstarts)==1 & isempty(tempends)==0 %if we have at least 1 end but no start, 1 must be the start
                tempstarts=1;
            end
            if isempty(tempstarts)==0 & isempty(tempends)==1 %if we have at least 1 start but no  end, it must end at the end of the track
                tempends=length(possvisitlocation);
            end
            if length(tempstarts)<length(tempends)
                 tempstarts=[1; tempstarts];
            end
            if length(tempstarts)>length(tempends)
                 tempends=[tempends; length(possvisitlocation)];
            end
            
      possstarts=[possstarts; tempstarts];
      possends=[possends; tempends];
      posslocation=[posslocation; repmat(lind,size(tempstarts))];
            
end

%Ensure that no proposed visit incorporates data from multiple flight
%segments
segmentstarts=NaN(size(DroneFlightSegments.SegmentNo));
segmentends=NaN(size(DroneFlightSegments.SegmentNo));
useg=unique(DroneFlightPaths.SegmentNo);
for sind=1:length(useg)
   thisseg=find(DroneFlightPaths.SegmentNo==useg(sind));
   segmentstarts(sind)=thisseg(1);
   segmentends(sind)=thisseg(end);
end
for pind=1:length(possstarts)
    segmentstartsmidvisit=find(segmentstarts>possstarts(pind) & segmentstarts<=possends(pind));
    if isempty(segmentstartsmidvisit)==0
        addstarts=segmentstarts(segmentstartsmidvisit);
        addends=segmentends(segmentstartsmidvisit);
        addends(end)=possends(pind);
        
        possends(pind)=segmentends(segmentstartsmidvisit(1)-1);
        possstarts=[possstarts; addstarts];
        possends=[possends; addends];
        posslocation=[posslocation; repmat(posslocation(pind), size(addstarts))];
    end
    
end

%Put potential visits into chronological order
[possstarts, posssort]=sort(possstarts);
possends=possends(posssort);
posslocation=posslocation(posssort);

%Accept potential visits that either contain convoluted flight whose CoM is
%within 50m of a location OR that stayed within 50m of the location for at
%least 21s
possduration=seconds(DroneFlightPaths.DateTime(possends)-DroneFlightPaths.DateTime(possstarts));
possanyconvolutedflight=zeros(size(possstarts));
for pind=1:length(possstarts)
   if any(DroneFlightPaths.ConvolutedSectionLocation(possstarts(pind):possends(pind)))>0
       possanyconvolutedflight(pind)=1;
   end
end
goodvisits=find(possduration>=DCATime | possanyconvolutedflight==1);
visitstarts=possstarts(goodvisits);
visitends=possends(goodvisits);
visitlocation=posslocation(goodvisits);

%%
%Add visit information to DroneFlightPaths
DroneFlightSegments.VisitCount=zeros(size(DroneFlightSegments.SegmentNo));
DroneFlightSegments.LocationsVisited=zeros(length(DroneFlightSegments.SegmentNo), nolocations);
DroneFlightPaths.VisitNoGlobal=zeros(size(DroneFlightPaths.SegmentNo));
DroneFlightPaths.VisitNoWithinSegment=zeros(size(DroneFlightPaths.SegmentNo));
DroneFlightPaths.VisitLocation=zeros(size(DroneFlightPaths.SegmentNo));
for sind=1:length(segmentstarts)
     visitsinsegment=find(visitstarts>=segmentstarts(sind) & visitstarts<=segmentends(sind)) ;
   if isempty(visitsinsegment)==0
    DroneFlightSegments.VisitCount(sind)=length(visitsinsegment);
     templocationsvisited=unique(visitlocation(visitsinsegment));
    DroneFlightSegments.LocationsVisited(sind,templocationsvisited)=1; 
        for vind=1:length(visitsinsegment)
            DroneFlightPaths.VisitNoGlobal(visitstarts(visitsinsegment(vind)):visitends(visitsinsegment(vind)))=repmat(visitsinsegment(vind), length(visitstarts(visitsinsegment(vind)):visitends(visitsinsegment(vind))), 1);
            DroneFlightPaths.VisitNoWithinSegment(visitstarts(visitsinsegment(vind)):visitends(visitsinsegment(vind)))=repmat(vind, length(visitstarts(visitsinsegment(vind)):visitends(visitsinsegment(vind))), 1);
            DroneFlightPaths.VisitLocation(visitstarts(visitsinsegment(vind)):visitends(visitsinsegment(vind)))=repmat(visitlocation(visitsinsegment(vind)), length(visitstarts(visitsinsegment(vind)):visitends(visitsinsegment(vind))), 1);
        end
   end
end

%%
%Details of each visit
 novisits=length(goodvisits);
DroneVisits.VisitNoGlobal=NaN(novisits,1);
DroneVisits.VisitNoWithinSegment=NaN(novisits,1);
DroneVisits.SegmentNo=NaN(novisits,1);
DroneVisits.BeeID=NaN(novisits,1);
DroneVisits.StartTime=repmat(datetime(2000,1,1,1,0,0), novisits,1);
DroneVisits.EndTime=repmat(datetime(2000,1,1,1,0,0), novisits,1);
DroneVisits.CoMX=NaN(novisits,1);
DroneVisits.CoMY=NaN(novisits,1);
DroneVisits.DistanceTotal=NaN(novisits,1);
DroneVisits.SpeedMean=NaN(novisits,1);
DroneVisits.SpeedSD=NaN(novisits,1);
DroneVisits.FlightPathIndices=cell(novisits,1);
DroneVisits=struct2table(DroneVisits);

for cind=1:novisits
     thisseg=visitstarts(cind):visitends(cind);
     tempx=DroneFlightPaths.X(thisseg);
     tempy=DroneFlightPaths.Y(thisseg);
     dx=diff(tempx);
     dy=diff(tempy);
     [~,dr]=cart2pol(tempx,tempy);
    DroneVisits.VisitNoGlobal(cind)=cind;
    DroneVisits.VisitNoWithinSegment(cind)=DroneFlightPaths.VisitNoWithinSegment(thisseg(1));
    DroneVisits.SegmentNo(cind)=DroneFlightPaths.SegmentNo(thisseg(1));
    DroneVisits.BeeID(cind)=DroneFlightPaths.BeeID(thisseg(1));
    DroneVisits.StartTime(cind)=DroneFlightPaths.DateTime(thisseg(1));
    DroneVisits.EndTime(cind)=DroneFlightPaths.DateTime(thisseg(end));
    DroneVisits.CoMX(cind)=mean(tempx);
    DroneVisits.CoMY(cind)=mean(tempy);
    DroneVisits.DistanceTotal(cind)=sum(dr);
    DroneVisits.SpeedMean(cind)=mean(DroneFlightPaths.SpeedStep(thisseg(2:end)));
    DroneVisits.SpeedSD(cind)=std(DroneFlightPaths.SpeedStep(thisseg(2:end)));
    DroneVisits.FlightPathIndices{cind}=thisseg;
end
    DroneVisits.Duration=DroneVisits.EndTime-DroneVisits.StartTime;
    DroneVisits.Location=visitlocation;
    DroneVisits.CongregationOrHive=zeros(size(DroneVisits.CoMX));
    DroneVisits.CongregationOrHive(DroneVisits.Location>0 & DroneVisits.Location<5)=1;
    DroneVisits.CongregationOrHive(DroneVisits.Location>=5)=2;


    DroneFlightSegments.DurationVisits=DroneFlightSegments.Duration;
useg=unique(DroneFlightPaths.SegmentNo);
for sind=1:length(useg)
   thisseg=find(DroneVisits.SegmentNo==useg(sind)); 
   DroneFlightSegments.DurationVisits(sind)=sum(DroneVisits.Duration(thisseg));
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% QUEENS
%%
%Flight Segment number nested within queen ID
[uqueens, ~, thisqueen]=unique(QueenFlightSegments.BeeID);
QueenFlightSegments.BeeFlightNo=NaN(size(QueenFlightSegments.SegmentNo));

for uind=1:length(uqueens)
    if uqueens(uind)<1000
    QueenFlightSegments.BeeFlightNo(thisqueen==uind)=(1:length(find(thisqueen==uind)))';
    end
end
%%
%Flight type: 0=neither starts or ends at hive; 1=complete flight, starts and
%ends at same hive; 2=starts at hive but return portion missing; 3=start
%portion missing but returns to hive
QueenFlightSegments.FlightType=NaN(size(QueenFlightSegments.SegmentNo));
QueenFlightSegments.FlightType(QueenFlightSegments.StartLocation==QueenFlightSegments.EndLocation & QueenFlightSegments.StartLocation>0 & QueenFlightSegments.EndLocation>0)=1;
QueenFlightSegments.FlightType(QueenFlightSegments.StartLocation>0 & QueenFlightSegments.StartLocation~=QueenFlightSegments.EndLocation)=2;
QueenFlightSegments.FlightType(QueenFlightSegments.StartLocation==0 & QueenFlightSegments.EndLocation>0)=3;
QueenFlightSegments.FlightType(QueenFlightSegments.StartLocation==0 & QueenFlightSegments.EndLocation==0)=0;

%%
%Work out GPS coordinates of all tracking data
[QueenFlightPaths.GPSLat, QueenFlightPaths.GPSLong]=ConvertCart2GPS(QueenFlightPaths.X, QueenFlightPaths.Y, Hives.X(1), Hives.Y(1), Hives.GPSLat(1), Hives.GPSLong(1));

%%
%Distances, times elapsed, speeds for each flight path
%Total distances, durations, start and end times, mean speeds for each flight segment; 
QueenFlightPaths.BeeID=NaN(size(QueenFlightPaths.SegmentNo));
QueenFlightPaths.SecondsCumulative=NaN(size(QueenFlightPaths.SegmentNo));
QueenFlightPaths.SecondsStep=NaN(size(QueenFlightPaths.SegmentNo));
QueenFlightPaths.DistanceStep=NaN(size(QueenFlightPaths.SegmentNo));
QueenFlightPaths.DistanceCumulative=NaN(size(QueenFlightPaths.SegmentNo));
QueenFlightPaths.SpeedStep=NaN(size(QueenFlightPaths.SegmentNo));
QueenFlightSegments.DataPointCount=NaN(size(QueenFlightSegments.SegmentNo));
QueenFlightSegments.StartTime=repmat(datetime(2000,1,1,1,0,0), size(QueenFlightSegments.SegmentNo));
QueenFlightSegments.EndTime=repmat(datetime(2000,1,1,1,0,0), size(QueenFlightSegments.SegmentNo));
QueenFlightSegments.Duration=NaN(size(QueenFlightSegments.SegmentNo));
QueenFlightSegments.DistanceTotal=NaN(size(QueenFlightSegments.SegmentNo));
QueenFlightSegments.DistanceMax=NaN(size(QueenFlightSegments.SegmentNo));
QueenFlightSegments.SpeedMean=NaN(size(QueenFlightSegments.SegmentNo));
QueenFlightSegments.SpeedSD=NaN(size(QueenFlightSegments.SegmentNo));
%
useg=unique(QueenFlightPaths.SegmentNo);
for sind=1:length(useg)
     thisseg=find(QueenFlightPaths.SegmentNo==useg(sind));
    QueenFlightPaths.BeeID(thisseg)=repmat(QueenFlightSegments.BeeID(useg(sind)), length(thisseg), 1);
    QueenFlightPaths.SecondsStep(thisseg)=[0; seconds(diff(QueenFlightPaths.DateTime(thisseg)))];
    QueenFlightPaths.SecondsCumulative(thisseg)=seconds(QueenFlightPaths.DateTime(thisseg)- QueenFlightPaths.DateTime(thisseg(1)));
     xdiff=[0; diff(QueenFlightPaths.X(thisseg))]  ; 
     ydiff=[0; diff(QueenFlightPaths.Y(thisseg))]  ; 
     [~,rdiff]=cart2pol(xdiff, ydiff);
    QueenFlightPaths.DistanceStep(thisseg)=rdiff;
    QueenFlightPaths.DistanceCumulative(thisseg)=cumsum(rdiff);
    QueenFlightPaths.SpeedStep(thisseg)=QueenFlightPaths.DistanceStep(thisseg)./QueenFlightPaths.SecondsStep(thisseg);
    QueenFlightSegments.DataPointCount(sind)=length(thisseg);
    QueenFlightSegments.DistanceTotal(sind)=QueenFlightPaths.DistanceCumulative(thisseg(end));
     xdiffstart=QueenFlightPaths.X(thisseg)-QueenFlightPaths.X(thisseg(1));
     ydiffstart=QueenFlightPaths.Y(thisseg)-QueenFlightPaths.Y(thisseg(1));
     [~, rdiffstart]=cart2pol(xdiffstart, ydiffstart);
    QueenFlightSegments.DistanceMax(sind)=max(rdiffstart);
    QueenFlightSegments.StartTime(sind)=QueenFlightPaths.DateTime(thisseg(1));
    QueenFlightSegments.EndTime(sind)=QueenFlightPaths.DateTime(thisseg(end));
    
    QueenFlightSegments.SpeedMean(sind)=mean(QueenFlightPaths.SpeedStep(thisseg(2:end)));
    QueenFlightSegments.SpeedSD(sind)=std(QueenFlightPaths.SpeedStep(thisseg(2:end)));
  
end
QueenFlightSegments.Duration=QueenFlightSegments.EndTime-QueenFlightSegments.StartTime;

%%
%Detect convoluted sections of each flightpath 

%Parameters for convoluted flight detection
vectorwindow=7*3; %21s, 3 revolutions of the radar
minstraightness=0.7; %minimum vector length
maxstepduration=12; %maximum gap permissible between datapoints
maxsteplength=40; %maximum movement permitted between consecutive datapoints
mindps=7; %minimum number of datapoints allowed in a convoluted section
stepandor=1; %1=only exclude datapoints that exceed BOTH maxstepduration and maxsteplength; 2=eclude datapoints that exceed EITHER maxstepduyration OR maxsteplength

%Create new variables to store convoluted leg data
QueenFlightPaths.ConvolutedSectionNoGlobal=zeros(size(QueenFlightPaths.SegmentNo));
QueenFlightPaths.ConvolutedSectionNoWithinSegment=zeros(size(QueenFlightPaths.SegmentNo));
QueenFlightSegments.ConvolutedSectionCount=NaN(size(QueenFlightSegments.SegmentNo));
cfgcounter=0;

useg=unique(QueenFlightPaths.SegmentNo);
for sind=1:length(useg)
     cfscounter=0;
     thisseg=find(QueenFlightPaths.SegmentNo==useg(sind));
     datainx=QueenFlightPaths.X(thisseg);
     datainy=QueenFlightPaths.Y(thisseg);
     dataint=QueenFlightPaths.SecondsCumulative(thisseg);
    %Find starts and ends of convoluted legs
    [convstartind, convendind]=DCAFlightDetector(datainx, datainy, dataint, vectorwindow, minstraightness, maxstepduration, maxsteplength, mindps, stepandor); 
    
    if isempty(convstartind)==0
        for cind=1:length(convstartind)
             cfgcounter=cfgcounter+1;
             cfscounter=cfscounter+1;
            QueenFlightPaths.ConvolutedSectionNoGlobal(thisseg(convstartind(cind):convendind(cind)))=cfgcounter;
            QueenFlightPaths.ConvolutedSectionNoWithinSegment(thisseg(convstartind(cind):convendind(cind)))=cfscounter;
        end
            QueenFlightSegments.ConvolutedSectionCount(sind)=cfscounter;
    else
        QueenFlightSegments.ConvolutedSectionCount(sind)=0;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Other functions used in data processing

%%
function [dcastartind, dcaendind]=DCAFlightDetector(datainx, datainy, dataint, vectorwindow, minstraightness, maxstepduration, maxsteplength, mindps, stepandor)

% Takes a spatial track and finds segments of convoluted motion.
% Simply, we first calculate the bearing of a straight line between every
% pair of consecutive dps. We calculate the resultant vector length (a
% scale-free  measure of straightness) over a 
% rolling window whose duration is given by 'vectorwindow'. Any dp that
% falls within a window whose straightness is greater than
% 'minstraightness'  is NOT part of a segment of convoluted motion. We can
% further restrict the remaining dtapoints by eleminating any transition
% between consecutive datapoints that lasts longer than a duration defined
% by 'maxstepduration' and/or covers a distance greater than 'maxsteplength'.
% Finally, any segment of convoluted track must contain at least N='mindps'
% datapoints.
% 'datainx' and 'datainy' are n*1 vectors of cartesian coordinates,
% presumed to be in the same units and giving positions of consecutive positional fixes.
% 'vectorwindow' and 'maxstepduration' defined in units of dataint (usually
% seconds).
% 'minstraightness' must be a number between 0:1 where 1 is a perfect
% straight line. Higher numbers mean fewer dps are excluded from segments
% of convoluted motion (since convoluted segments are those that do not
% meet a given standard of straightness).
% maxsteplength defined in units of datainx/y (usually m).
% 'stepandor' determines whether restrictions on step duration and distance
% are applied as AND or OR conditions. 1=AND: datapints only excluded if
% they exceed both maxstepduration and maxsteplength; 2=OR: datapoints
% excluded if either maxstepduration or maxsteplength are exceeded.
% Outputs are 'dcastartind' and 'dcaendind': m*1 vectors of equal length
% containing the index of the first and last datapoint in every segment of
% convoluted motion. There are no restrictions on how many segments of
% convoluted motion can appear in a track, so the value of m varies
% depending on the properties of the track.
%%
%defaults
if nargin<3 | isempty(dataint)==1
dataint=(1:length(datainx))';
end

if nargin<4 | isempty(vectorwindow)==1
vectorwindow=24;
end

if nargin<5 | isempty(minstraightness)==1
minstraightness=0.7;
end

%If no value is entered for maxstepduration we automatically set it to be
%larger than any real step  duration in the dataset. This is equivalent to
%turning it off entirely. I.e. we only restrict the data based on step
%duration if a value for maxstepduration is entered
if nargin<6 | isempty(maxstepduration)==1
maxstepduration=max(diff(dataint))+1;
end

%If no value is entered for maxsteplength we automatically set it to be
%larger than any real step distance in the dataset. This is equivalent to
%turning it off entirely. I.e. we only restrict the data based on step
%distance if a value for maxsteplength is entered
if nargin<7 | isempty(maxsteplength)==1
    [~, dt]=cart2pol(diff(datainx),diff(datainy));
maxsteplength=max(dt)+1;
end

%If no value is entered for mindps, we set it to 1. I.e. any datapoints
%that are not excluded by reason of straightness, step duration or
%stepdistance will be returned as part of a segment of convoluted movement.
if nargin<8 | isempty(mindps)==1
mindps=1;
end

if nargin<6 | isempty(stepandor)==1 %when excluding steps, we either can exclude any step that meets the threshold for EITHER duration OR distance; alternatively we can only exclude steps that meet thresholds for BOTH duration AND distance
stepandor=1; %1=AND, 2=OR
end

%%
    
    if dataint(end)-dataint(1)>=vectorwindow  
        %find all dps that fall into a window of suitably straight flight
        [bearingveclength, bearingstarts, bearingends, ~]=WindowVecLength(datainx, datainy, dataint, vectorwindow);
 %%
        straightdp=zeros(size(datainx));
        if isempty(bearingveclength)==0
        straightind=find(bearingveclength>=minstraightness);
            if isempty(straightind)==0
                for qind=1:length(straightind)
                    straightdp(bearingstarts(straightind(qind)):bearingends(straightind(qind)))=1;
                end 
            end
        end
        %find first and last dp of each segment of non-DCA flight
        straightstatechange=[diff(straightdp); 0];
%%        
        %Find all transitions that exceed the maximum allowed step duration
        dt=[0; diff(dataint)];
        durstep=zeros(size(datainx));
        durstep(dt>maxstepduration)=1;
%%        
        %Find all transitions that exceed the maximum allowed steplength
        dx=[0; diff(datainx)]; dy=[0; diff(datainy)];
        [~, dr]=cart2pol(dx,dy);
        diststep=zeros(size(datainx));
        diststep(dr>maxsteplength)=1;
%%
        %Combine distance and duration to determine which transitions are
        %unacceptable
        if stepandor==2
        longstep=durstep;
        longstep(diststep==1)=1;
        else
        longstep=zeros(size(datainx));
        longstep(durstep==1 & diststep==1)=1;
        end
%%        
        %find start and end points of non-straight segments
         startptsnonstraight=zeros(size(datainx));
        startptsnonstraight(([0; straightstatechange(1:end-1)])==-1)=1;
        if straightdp(1)==0; startptsnonstraight(1)=1; end
         endptsnonstraight=zeros(size(datainx));
        endptsnonstraight(straightstatechange==1)=1;
        if straightdp(end)==0; endptsnonstraight(end)=1; end
            
        %find start and end points of segments that do not have unacceptable gaps 
         startptsnonlong=zeros(size(datainx));
        startptsnonlong(longstep==1 & straightdp==0)=1;
         endptsnonlong=zeros(size(datainx));
        endptsnonlong(([longstep(2:end); 0])==1 & straightdp==0)=1;
        
        %combine startpts and endpts
         startpts=startptsnonstraight;
        startpts(startptsnonlong==1)=1;
         endpts=endptsnonstraight;
        endpts(endptsnonlong==1)=1;
        
        %find indices of start and ends
        dcastarts=find(startpts==1);
        dcaends=find(endpts==1);

        %check length of dca segments
        dcalength=(dcaends-dcastarts)+1;

        %two ways to deal with short DCA segments: 1) just delete them; 2)
        %check for several short segments separated by a short straight segment
        %and amalgamate them. All straight segments must be at least
        %'windowsize' long anyway. How long does a straight segment have to be
        %before it can no longer be amalgamated into a piece of DCA flight?
        %For now, we will just ignore short DCA
        dcastartind=dcastarts(dcalength>=mindps);
        dcaendind=dcaends(dcalength>=mindps);

    else %not enough data
        dcastartind=[];
        dcaendind=[];
    end %if dataint(end)-dataint(1)>=windowsize 
    
end %function [dcastartind, dcaendind]=DCAFlightDetector(datainx, datainy, dataint, vectorwindow, longstepwindow, minstraightness, maxsteplength, mindps)


function [bearingveclength, bearingstarts, bearingends, meanveclength]=WindowVecLength(datainx, datainy, dataint, vectorwindow)
% calculate resultant vector length over a certain window (newer version): 
%vector length: larger score = straighter path   
%step ratio: larger score = straighter path
%bearingveclength: 1= straight line, 0=circle
%stepratio: 1=dist travelled in one step is infinitessimally small fraction of dist travelled over whole window (straighter path); 0= dist travelled over 1 step is as large or larger than dist travelled over whole window (bee not moving away, unstraight path) 
%meanveclength: mean vector length from all windows - a measure of overall
%path straightness (1=perfectly straight, 0=circle)
if nargin<3 | isempty(vectorwindow)==1
    vectorwindow=24;
end

%Work  out bearings (angle between each pair of datapoints
dxsegment=[datainx(2:end); 0]-datainx;
dysegment=[datainy(2:end); 0]-datainy;
dtsegment=[0; dataint(2:end)-dataint(1:end-1)];
[bearingsoversegment, ~]=cart2pol(dxsegment, dysegment);

%For each dp, work out the last point of a window whose duration is
%vectorwindow
tsegsum=cumsum(dtsegment);
windowendind=zeros(size(dtsegment));
for wind=1:length(windowendind)
   tpassed=tsegsum-tsegsum(wind);
   tempwend=find(tpassed<=vectorwindow, 1, 'last');
   if isempty(tempwend)==0
       windowendind(wind)=tempwend;
   end
end
windowendind(windowendind==0)=length(windowendind);
lastgoodwindow=find(windowendind==max(windowendind),1);

%For each window, caclulate the resultant vector length of all the bearings
%in that window
bearingveclength=zeros(size(datainx));
for bind=1:lastgoodwindow
    bearingveclength(bind)=circ_r(bearingsoversegment(bind:windowendind(bind)));
end
[~, goodbearings, ~]=unique(windowendind);

%Occasionally, it may be the case that missing data or other reasons mean
%that some windows are subsets of other windows (e.g. every datapoint i the
%window was also included in a previous window). These are eliminated.
meanveclength=mean(bearingveclength(goodbearings));
bearingstarts=goodbearings;
bearingends=windowendind(goodbearings);
bearingveclength=bearingveclength(goodbearings);
end %function [bearingveclength, stepratio]=StepScore(datainx, datainy, windowsize)

%%
function [GPSLat, GPSLong]=ConvertCart2GPS(InX, InY, RefX, RefY, RefLat, RefLong)
%Assumes that InX are data along E-W axis and InY are data along N-S  axis.
%Convert ref to digital degrees
if size(RefLat,2)==1
    gpstype=1; 
elseif size(RefLat,2)==2
    gpstype=2;
    RefLat=dm2degrees(RefLat);
    RefLong=dm2degrees(RefLong);
elseif size(RefLat,2)==3
    gpstype=3;
    RefLat=dms2degrees(RefLat);
    RefLong=dms2degrees(RefLong);
end

relativex=InX-RefX;
relativey=InY-RefY;
[relativetheta, relativer]=cart2pol(relativex, relativey);
relativedeg=ModAnglesJLWv2(AngleConvertMlab2Radar(rad2ang(relativetheta)));
GPSLat=zeros(length(InX),1);
GPSLong=zeros(length(InX),1);
for dpind=1:length(InX)
    [lat2,lon2] = vreckon(RefLat, RefLong, relativer(dpind), relativedeg(dpind));
GPSLat(dpind)=lat2;  
GPSLong(dpind)=lon2;
end %for dpind=1:length(DataRange)
GPSLong(GPSLong>180)=GPSLong(GPSLong>180)-360; %The vreckon function gives outputs in degrees East, but most mapping programs only understand degrees in the form -180:+180 degrees. For this reason we must convert any longitude greater than 180 into negative degrees from greenwich

%Convert output to correct format
if gpstype==2
    GPSLat=degrees2dm(GPSLat);
    GPSLong=degrees2dm(GPSLong);
elseif gpstype==3
    GPSLat=degrees2dms(GPSLat);
    GPSLong=degrees2dms(GPSLong);
end

end %[GPSLat, GPSLong]=ConvertCart2GPS(InX, InY, RefX, RefY, RefLat, RefLong)

%%
function [fixedangles]=ModAnglesJLWv2(rawangles)
%This function finds the
%modulus of an input vector of angles and returns them in the format +/-180
%degrees

angles=rawangles;
toolowindex=find(angles<-180);
toohighindex=find(angles>180);

while isempty(toolowindex)==0 || isempty(toohighindex)==0
angles(toolowindex)=angles(toolowindex)+360;
angles(toohighindex)=angles(toohighindex)-360;

toohighindex=find(angles>180);
toolowindex=find(angles<-180);
end
fixedangles=angles;
end  %function [fixedangles]=ModAnglesJLWv2(rawangles)

%%
function thetaradar=AngleConvertMlab2Radar(thetamlab)
%converts the azimuth angles outputted from the TAS radar program into
%angles in matlab format (radar has 0 at North compass point and increases clockwise, matlab standard has 0 at East compass point and increases anticlockwise)
%Input and output should both be in degrees.

thetaradar=mod((90-thetamlab),360);
end %function thetaradar=AngleConvertMlab2Radar(thetamlab)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Functions written by other people. Reproduced here so that this code will run even for people who do not have these functions. Sources noted in comments to each function

%%
function alpha = rad2ang(alpha)
%
% Circular Statistics Toolbox for Matlab
% copyright (c) 2006 philipp berens
% berens@tuebingen.mpg.de - www.kyb.mpg.de/~berens
% distributed under GPL with no liability
% http://www.gnu.org/copyleft/gpl.html

alpha = alpha / pi *180;
end %function alpha = rad2ang(alpha)

%%
function alpha = ang2rad(alpha)
%
% Circular Statistics Toolbox for Matlab
% copyright (c) 2006 philipp berens
% berens@tuebingen.mpg.de - www.kyb.mpg.de/~berens
% distributed under GPL with no liability
% http://www.gnu.org/copyleft/gpl.html

alpha = alpha * pi /180;
end %function alpha = ang2rad(alpha)

%%
function r = circ_r(alpha, w, d, dim)
% r = circ_r(alpha, w, d)
%   Computes mean resultant vector length for circular data.
%
%   Input:
%     alpha	sample of angles in radians
%     [w		number of incidences in case of binned angle data]
%     [d    spacing of bin centers for binned data, if supplied 
%           correction factor is used to correct for bias in 
%           estimation of r, in radians (!)]
%     [dim  compute along this dimension, default is 1]
%
%     If dim argument is specified, all other optional arguments can be
%     left empty: circ_r(alpha, [], [], dim)
%
%   Output:
%     r		mean resultant length
%
% PHB 7/6/2008
%
% References:
%   Statistical analysis of circular data, N.I. Fisher
%   Topics in circular statistics, S.R. Jammalamadaka et al. 
%   Biostatistical Analysis, J. H. Zar
%
% Circular Statistics Toolbox for Matlab

% By Philipp Berens, 2009
% berens@tuebingen.mpg.de - www.kyb.mpg.de/~berens/circStat.html

if nargin < 4
  dim = 1;
end

if nargin < 2 || isempty(w) 
  % if no specific weighting has been specified
  % assume no binning has taken place
	w = ones(size(alpha));
else
  if size(w,2) ~= size(alpha,2) || size(w,1) ~= size(alpha,1) 
    error('Input dimensions do not match');
  end 
end

if nargin < 3 || isempty(d)
  % per default do not apply correct for binned data
  d = 0;
end

% compute weighted sum of cos and sin of angles
r = sum(w.*exp(1i*alpha),dim);

% obtain length 
r = abs(r)./sum(w,dim);

% for data with known spacing, apply correction factor to correct for bias
% in the estimation of r (see Zar, p. 601, equ. 26.16)
if d ~= 0
  c = d/2/sin(d/2);
  r = c*r;
end
end %function r = circ_r(alpha, w, d, dim)

%%
function [lat2,lon2,a21] = vreckon(lat1,lon1,s,a12)
% RECKON - Using the WGS-84 Earth ellipsoid, travel a given distance along
%          a given azimuth starting at a given initial point, and return
%          the endpoint within a few millimeters of accuracy, using
%          Vincenty's algorithm.
%
% USAGE:
% [lat2,lon2] = vreckon(lat1, lon1, s, a12)
%
% VARIABLES:
% lat1 = inital latitude (degrees)
% lon1 = initial longitude (degrees)
% s    = distance (meters)
% a12  = intial azimuth (degrees)
% lat2, lon2 = second point (degrees)
% a21  = reverse azimuth (degrees), at final point facing back toward the
%        intial point
%
% Original algorithm source:
% T. Vincenty, "Direct and Inverse Solutions of Geodesics on the Ellipsoid
% with Application of Nested Equations", Survey Review, vol. 23, no. 176,
% April 1975, pp 88-93.
% Available at: http://www.ngs.noaa.gov/PUBS_LIB/inverse.pdf
%
% Notes: 
% (1) The Vincenty reckoning algorithm was transcribed verbatim into
%     JavaScript by Chris Veness. It was modified and translated to Matlab
%     by Michael Kleder. Mr. Veness's website is:
%     http://www.movable-type.co.uk/scripts/latlong-vincenty-direct.html
% (2) Error correcting code, polar error corrections, WGS84 ellipsoid
%     parameters, testing, and comments by Michael Kleder.
% (3) By convention, when starting at a pole, the longitude of the initial
%     point (otherwise meaningless) determines the longitude line along
%     which to traverse, and hence the longitude of the final point.
% (4) The convention noted in (3) above creates a discrepancy with VDIST
%     when the the intial or final point is at a pole. In the VDIST
%     function, when traversing from a pole, the azimuth is  0 when
%     heading away from the south pole and 180 when heading away from the
%     north pole. In contrast, this VRECKON function uses the azimuth as
%     noted in (3) above when traversing away form a pole.
% (5) In testing, where the traversal subtends no more than 178 degrees,
%     this function correctly inverts the VDIST function to within 0.2
%     millimeters of distance, 5e-10 degrees of forward azimuth,
%     and 5e-10 degrees of reverse azimuth. Precision reduces as test
%     points approach antipodal because the precision of VDIST is reduced
%     for nearly antipodal points. (A warning is given by VDIST.)
% (6) Tested but no warranty. Use at your own risk.
% (7) Ver 1.0, Michael Kleder, November 2007

% Input check:
if abs(lat1)>90
    error('Input latitude must be between -90 and 90 degrees, inclusive.')
end
a = 6378137; % semimajor axis
b = 6356752.31424518; % semiminor axis
f = 1/298.257223563; % flattening coefficient
lat1   = lat1 * .1745329251994329577e-1; % intial latitude in radians
lon1   = lon1 * .1745329251994329577e-1; % intial longitude in radians
% correct for errors at exact poles by adjusting 0.6 millimeters:
kidx = abs(pi/2-abs(lat1)) < 1e-10;
if any(kidx);
    lat1(kidx) = sign(lat1(kidx))*(pi/2-(1e-10));
end
alpha1 = a12 * .1745329251994329577e-1; % inital azimuth in radians
sinAlpha1 = sin(alpha1);
cosAlpha1 = cos(alpha1);
tanU1 = (1-f) * tan(lat1);
cosU1 = 1 / sqrt(1 + tanU1*tanU1);
sinU1 = tanU1*cosU1;
sigma1 = atan2(tanU1, cosAlpha1);
sinAlpha = cosU1 * sinAlpha1;
cosSqAlpha = 1 - sinAlpha*sinAlpha;
uSq = cosSqAlpha * (a*a - b*b) / (b*b);
A = 1 + uSq/16384*(4096+uSq*(-768+uSq*(320-175*uSq)));
B = uSq/1024 * (256+uSq*(-128+uSq*(74-47*uSq)));
sigma = s / (b*A);
sigmaP = 2*pi;
while (abs(sigma-sigmaP) > 1e-12)
    cos2SigmaM = cos(2*sigma1 + sigma);
    sinSigma = sin(sigma);
    cosSigma = cos(sigma);
    deltaSigma = B*sinSigma*(cos2SigmaM+B/4*(cosSigma*(-1+...
        2*cos2SigmaM*cos2SigmaM)-...
        B/6*cos2SigmaM*(-3+4*sinSigma*sinSigma)*(-3+...
        4*cos2SigmaM*cos2SigmaM)));
    sigmaP = sigma;
    sigma = s / (b*A) + deltaSigma;
end
tmp = sinU1*sinSigma - cosU1*cosSigma*cosAlpha1;
lat2 = atan2(sinU1*cosSigma + cosU1*sinSigma*cosAlpha1,...
    (1-f)*sqrt(sinAlpha*sinAlpha + tmp*tmp));
lambda = atan2(sinSigma*sinAlpha1, cosU1*cosSigma - ...
    sinU1*sinSigma*cosAlpha1);
C = f/16*cosSqAlpha*(4+f*(4-3*cosSqAlpha));
L = lambda - (1-C) * f * sinAlpha * (sigma + C*sinSigma*(cos2SigmaM+...
    C*cosSigma*(-1+2*cos2SigmaM*cos2SigmaM)));
lon2 = lon1 + L;
% output degrees
lat2 = lat2 * 57.295779513082322865;
lon2 = lon2 * 57.295779513082322865;
lon2 = mod(lon2,360); % follow [0,360] convention
if nargout > 2
    a21 = atan2(sinAlpha, -tmp); 
    a21  = 180 + a21  * 57.295779513082322865; % note direction reversal
    a21=mod(a21,360);
end
return
end %function [lat2,lon2,a21] = vreckon(lat1,lon1,s,a12)